
const IO = require('../io.js');
const IMem = require('./imem.js');
const ROM = require('./rom.js');


class RAM extends ROM {
  constructor(pages){
    super(pages);
  }

  get pages(){return this.__map.length/256;}
  get size(){return this.__map.length;}
  get writable(){return true;}


  get address(){return this.__addr;}
  set address(a){
    this.__addr = Math.min(this.__map.length - 1, Math.max(0, a));
  }  

  get byte(){
    this.__io.triggerRead(this.__addr);
    return this.__map[this.__addr];
  }
  set byte(b){
    this.__map[this.__addr] = b & 0xFF;
    this.__io.triggerWrite(this.__addr, b & 0xFF);
  }

  onAddressWrite(addr, fn){
    if (addr < 0 || addr >= this.size)
      throw new RangeError("Memory address is out of bounds.");
    this.__io.onWrite(addr, fn);
    return this;
  }

  poke(a, b){
    a = Math.min(this.__map.length - 1, Math.max(0, a));
    this.__map[a] = b & 0xFF;
    return this;
  }
}



module.exports = RAM;




